@@ -1,9 +1,11 @@
+# pylint: disable=too-many-lines
 import enum
 import os
 import sys
 import traceback
 from collections import defaultdict
 from enum import Enum
+from queue import Queue
 from typing import Any, Callable, Dict, Generator, Iterable, List, Optional, Tuple, Union
 from urllib.parse import urlparse
 
@@ -14,7 +16,7 @@ import yaml
 
 from .. import checks as checks_module
 from .. import fixups as _fixups
-from .. import runner
+from .. import runner, service
 from .. import targets as targets_module
 from ..constants import (
     DEFAULT_DATA_GENERATION_METHODS,
@@ -26,7 +28,7 @@ from ..constants import (
 from ..exceptions import HTTPError
 from ..fixups import ALL_FIXUPS
 from ..hooks import GLOBAL_HOOK_DISPATCHER, HookContext, HookDispatcher, HookScope
-from ..models import CheckFunction
+from ..models import Case, CheckFunction
 from ..runner import events, prepare_hypothesis_settings
 from ..schemas import BaseSchema
 from ..specs.graphql import loaders as gql_loaders
@@ -34,11 +36,11 @@ from ..specs.graphql.schemas import GraphQLSchema
 from ..specs.openapi import loaders as oas_loaders
 from ..stateful import Stateful
 from ..targets import Target
-from ..types import Filter
-from ..utils import file_exists, get_requests_auth, import_app
+from ..types import Filter, RequestCert
+from ..utils import GenericResponse, file_exists, get_requests_auth, import_app
 from . import callbacks, cassettes, output
 from .constants import DEFAULT_WORKERS, MAX_WORKERS, MIN_WORKERS
-from .context import ExecutionContext
+from .context import ExecutionContext, ServiceContext
 from .debug import DebugOutputHandler
 from .handlers import EventHandler
 from .junitxml import JunitXMLHandler
@@ -127,7 +129,7 @@ class DeprecatedOption(click.Option):
 @click.option("--pre-run", help="A module to execute before the running the tests.", type=str)
 @click.version_option()
 def schemathesis(pre_run: Optional[str] = None) -> None:
-    """Command line tool for testing your web application built with Open API / Swagger specifications."""
+    """Command line tool for testing your web application built with Open API / GraphQL specifications."""
     if pre_run:
         load_hook(pre_run)
 
@@ -241,7 +243,7 @@ class GroupedOption(click.Option):
     "--header",
     "-H",
     "headers",
-    help=r"Custom header in a that will be used in all requests to the server. Example: Authorization: Bearer\ 123",
+    help=r"Custom header that will be used in all requests to the server. Example: Authorization: Bearer\ 123",
     multiple=True,
     type=str,
     callback=callbacks.validate_headers,
@@ -326,6 +328,23 @@ class GroupedOption(click.Option):
     show_default=True,
     callback=callbacks.convert_request_tls_verify,
 )
+@click.option(
+    "--request-cert",
+    help="File path of unencrypted client certificate for authentication. "
+    "The certificate can be bundled with a private key (e.g. PEM) or the private "
+    "key can be provided with the --request-cert-key argument.",
+    type=click.Path(exists=True),
+    default=None,
+    show_default=False,
+)
+@click.option(
+    "--request-cert-key",
+    help="File path of the private key of the client certificate.",
+    type=click.Path(exists=True),
+    default=None,
+    show_default=False,
+    callback=callbacks.validate_request_cert_key,
+)
 @click.option(
     "--validate-schema",
     help="Enable or disable validation of input schema.",
@@ -458,6 +477,17 @@ class GroupedOption(click.Option):
     group=ParameterGroup.hypothesis,
 )
 @click.option("--no-color", help="Disable ANSI color escape codes.", type=bool, is_flag=True)
+@click.option(
+    "--schemathesis-io-token",
+    help="Schemathesis.io authentication token. If present, test run results will be uploaded to Schemathesis.io",
+    type=str,
+)
+@click.option(
+    "--schemathesis-io-url",
+    help="Schemathesis.io base URL.",
+    default=service.DEFAULT_URL,
+    type=str,
+)
 @click.option("--verbosity", "-v", help="Reduce verbosity of error output.", count=True)
 @click.pass_context
 def run(
@@ -481,6 +511,8 @@ def run(
     app: Optional[str] = None,
     request_timeout: Optional[int] = None,
     request_tls_verify: bool = True,
+    request_cert: Optional[str] = None,
+    request_cert_key: Optional[str] = None,
     validate_schema: bool = True,
     skip_deprecated_operations: bool = False,
     junit_xml: Optional[click.utils.LazyFile] = None,
@@ -502,10 +534,12 @@ def run(
     hypothesis_verbosity: Optional[hypothesis.Verbosity] = None,
     verbosity: int = 0,
     no_color: bool = False,
+    schemathesis_io_token: Optional[str] = None,
+    schemathesis_io_url: str = service.DEFAULT_URL,
 ) -> None:
     """Perform schemathesis test against an API specified by SCHEMA.
 
-    SCHEMA must be a valid URL or file path pointing to an Open API / Swagger specification.
+    SCHEMA must be a valid URL or file path pointing to an Open API / GraphQL specification.
     """
     # pylint: disable=too-many-locals
     maybe_disable_color(ctx, no_color)
@@ -540,6 +574,7 @@ def run(
         data_generation_methods=data_generation_methods,
         force_schema_version=force_schema_version,
         request_tls_verify=request_tls_verify,
+        request_cert=prepare_request_cert(request_cert, request_cert_key),
         auth=auth,
         auth_type=auth_type,
         headers=headers,
@@ -570,9 +605,17 @@ def run(
         verbosity,
         code_sample_style,
         debug_output_file,
+        schemathesis_io_token,
+        schemathesis_io_url,
     )
 
 
+def prepare_request_cert(cert: Optional[str], key: Optional[str]) -> Optional[RequestCert]:
+    if cert is not None and key is not None:
+        return cert, key
+    return cert
+
+
 @attr.s(slots=True)
 class LoaderConfig:
     """Container for API loader parameters.
@@ -588,6 +631,7 @@ class LoaderConfig:
     data_generation_methods: Tuple[DataGenerationMethod, ...] = attr.ib()  # pragma: no mutate
     force_schema_version: Optional[str] = attr.ib()  # pragma: no mutate
     request_tls_verify: Union[bool, str] = attr.ib()  # pragma: no mutate
+    request_cert: Optional[RequestCert] = attr.ib()  # pragma: no mutate
     # Network request parameters
     auth: Optional[Tuple[str, str]] = attr.ib()  # pragma: no mutate
     auth_type: Optional[str] = attr.ib()  # pragma: no mutate
@@ -609,6 +653,7 @@ def into_event_stream(
     data_generation_methods: Tuple[DataGenerationMethod, ...],
     force_schema_version: Optional[str],
     request_tls_verify: Union[bool, str],
+    request_cert: Optional[RequestCert],
     # Network request parameters
     auth: Optional[Tuple[str, str]],
     auth_type: Optional[str],
@@ -644,6 +689,7 @@ def into_event_stream(
             data_generation_methods=data_generation_methods,
             force_schema_version=force_schema_version,
             request_tls_verify=request_tls_verify,
+            request_cert=request_cert,
             auth=auth,
             auth_type=auth_type,
             headers=headers,
@@ -660,6 +706,7 @@ def into_event_stream(
             headers=headers,
             request_timeout=request_timeout,
             request_tls_verify=request_tls_verify,
+            request_cert=request_cert,
             seed=seed,
             exit_first=exit_first,
             dry_run=dry_run,
@@ -769,6 +816,8 @@ def get_graphql_loader_kwargs(
 
 def _add_requests_kwargs(kwargs: Dict[str, Any], config: LoaderConfig) -> None:
     kwargs["verify"] = config.request_tls_verify
+    if config.request_cert is not None:
+        kwargs["cert"] = config.request_cert
     if config.auth is not None:
         kwargs["auth"] = get_requests_auth(config.auth, config.auth_type)
 
@@ -820,6 +869,8 @@ def execute(
     verbosity: int,
     code_sample_style: CodeSampleStyle,
     debug_output_file: Optional[click.utils.LazyFile],
+    schemathesis_io_token: Optional[str],
+    schemathesis_io_url: str,
 ) -> None:
     """Execute a prepared runner by drawing events from it and passing to a proper handler."""
     handlers: List[EventHandler] = []
@@ -827,6 +878,11 @@ def execute(
         handlers.append(JunitXMLHandler(junit_xml))
     if debug_output_file is not None:
         handlers.append(DebugOutputHandler(debug_output_file))
+    service_context = None
+    if schemathesis_io_token is not None:
+        service_queue: Queue = Queue()
+        service_context = ServiceContext(url=schemathesis_io_url, queue=service_queue)
+        handlers.append(service.ServiceReporter(service_queue, schemathesis_io_token, schemathesis_io_url))
     if store_network_log is not None:
         # This handler should be first to have logs writing completed when the output handler will display statistic
         handlers.append(cassettes.CassetteWriter(store_network_log))
@@ -839,6 +895,7 @@ def execute(
         junit_xml_file=junit_xml.name if junit_xml is not None else None,
         verbosity=verbosity,
         code_sample_style=code_sample_style,
+        service=service_context,
     )
 
     def shutdown() -> None:
@@ -927,3 +984,34 @@ def after_init_cli_run_handlers(
 
     Might be used to add extra event handlers.
     """
+
+
+@HookDispatcher.register_spec([HookScope.GLOBAL])
+def before_call(context: HookContext, case: Case) -> None:
+    """Called before every network call in CLI tests.
+
+    Use cases:
+     - Modification of `case`. For example, adding some pre-determined value to its query string.
+     - Logging
+    """
+
+
+@HookDispatcher.register_spec([HookScope.GLOBAL])
+def after_call(context: HookContext, case: Case, response: GenericResponse) -> None:
+    """Called after every network call in CLI tests.
+
+    Note that you need to modify the response in-place.
+
+    Use cases:
+     - Response post-processing, like modifying its payload.
+     - Logging
+    """
+
+
+@HookDispatcher.register_spec([HookScope.GLOBAL])
+def process_call_kwargs(context: HookContext, case: Case, kwargs: Dict[str, Any]) -> None:
+    """Called before every network call in CLI tests.
+
+    Aims to modify the argument passed to `case.call` / `case.call_wsgi` / `case.call_asgi`.
+    Note that you need to modify `kwargs` in-place.
+    """
