@@ -2,24 +2,33 @@ import base64
 import os
 import platform
 import shutil
-from typing import Any, Dict, List, Optional, Tuple, Union, cast
+import time
+from queue import Queue
+from typing import Any, Dict, Generator, List, Optional, Tuple, Union, cast
 
 import click
+import requests
 from hypothesis import settings
 
+from ... import service
 from ..._compat import metadata
 from ...constants import CodeSampleStyle, __version__
 from ...models import Response, Status
 from ...runner import events
-from ...runner.serialization import SerializedCase, SerializedError, SerializedTestResult
+from ...runner.serialization import SerializedCase, SerializedError, SerializedTestResult, deduplicate_failures
 from ..context import ExecutionContext
-from ..handlers import EventHandler, get_unique_failures
+from ..handlers import EventHandler
 
 DISABLE_SCHEMA_VALIDATION_MESSAGE = (
     "\nYou can disable input schema validation with --validate-schema=false "
     "command-line option\nIn this case, Schemathesis cannot guarantee proper"
     " behavior during the test run"
 )
+ISSUE_TRACKER_URL = (
+    "https://github.com/schemathesis/schemathesis/issues/new?"
+    "labels=Status%3A+Review+Needed%2C+Type%3A+Bug&template=bug_report.md&title=%5BBUG%5D"
+)
+SPINNER_REPETITION_NUMBER = 10
 
 
 def get_terminal_width() -> int:
@@ -180,7 +189,7 @@ def display_failures(context: ExecutionContext, event: events.Finished) -> None:
 def display_failures_for_single_test(context: ExecutionContext, result: SerializedTestResult) -> None:
     """Display a failure for a single method / path."""
     display_subsection(result)
-    checks = get_unique_failures(result.checks)
+    checks = deduplicate_failures(result.checks)
     for idx, check in enumerate(checks, 1):
         message: Optional[str]
         if check.message:
@@ -254,17 +263,115 @@ def display_statistic(context: ExecutionContext, event: events.Finished) -> None
     if total:
         display_checks_statistics(total)
 
-    if context.cassette_file_name or context.junit_xml_file:
-        click.echo()
-
     if context.cassette_file_name:
+        click.echo()
         category = click.style("Network log", bold=True)
         click.secho(f"{category}: {context.cassette_file_name}")
 
     if context.junit_xml_file:
+        click.echo()
         category = click.style("JUnit XML file", bold=True)
         click.secho(f"{category}: {context.junit_xml_file}")
 
+    handle_service_integration(context)
+
+
+def handle_service_integration(context: ExecutionContext) -> None:
+    """If Schemathesis.io integration is enabled, wait for the handler & print the resulting status."""
+    if context.service:
+        click.echo()
+        title = click.style("Schemathesis.io", bold=True)
+        event = wait_for_service_handler(context.service.queue, title)
+        color = {
+            service.Completed: "green",
+            service.Error: "red",
+            service.Timeout: "red",
+        }[event.__class__]
+        status = click.style(event.name, fg=color, bold=True)
+        click.echo(f"{title}: {status}\r", nl=False)
+        click.echo()
+        if isinstance(event, service.Completed):
+            report_title = click.style("Report", bold=True)
+            click.echo(f"{report_title}: {event.short_url}")
+        if isinstance(event, service.Error):
+            display_service_error(event, context)
+
+
+def display_service_error(event: service.Error, context: ExecutionContext) -> None:
+    """Show information about an error during communication with Schemathesis.io."""
+    click.echo()
+    if isinstance(event.exception, requests.HTTPError):
+        status_code = event.exception.response.status_code
+        click.secho(f"Schemathesis.io responded with HTTP {status_code}", fg="red")
+        if 500 <= status_code <= 599:
+            # Server error, should be resolved soon
+            click.secho(
+                "It is likely that we are already notified about the issue and working on a fix\n"
+                "Please, try again in 30 minutes",
+                fg="red",
+            )
+        elif status_code == 401:
+            # Likely an invalid token
+            click.secho(
+                "Please, check that you use the proper CLI upload token\n"
+                "See https://schemathesis.readthedocs.io/en/stable/service.html for more details",
+                fg="red",
+            )
+        else:
+            # Other client-side errors are likely caused by a bug on the CLI side
+            ask_to_report(event, context)
+    elif isinstance(event.exception, requests.RequestException):
+        ask_to_report(event, context, report_to_issues=False)
+    else:
+        ask_to_report(event, context)
+
+
+def ask_to_report(
+    event: service.Error, context: ExecutionContext, report_to_issues: bool = True, extra: str = ""
+) -> None:
+    # Likely an internal Schemathesis error
+    message = event.get_message(context.show_errors_tracebacks)
+    if isinstance(event.exception, requests.RequestException) and event.exception.response is not None:
+        response = f"Response: {event.exception.response.text}"
+    else:
+        response = ""
+    if report_to_issues:
+        ask = f"Please, consider reporting it to our issue tracker: {ISSUE_TRACKER_URL}\n"
+    else:
+        ask = ""
+    click.secho(
+        f"An error happened during uploading reports to Schemathesis.io:\n"
+        f"{extra}"
+        f"{ask}"
+        f"{response}"
+        f"\n    {message.strip()}",
+        fg="red",
+    )
+
+
+def wait_for_service_handler(queue: Queue, title: str) -> service.Event:
+    """Wait for the Schemathesis.io handler to finish its job."""
+    start = time.monotonic()
+    spinner = create_spinner(SPINNER_REPETITION_NUMBER)
+    # The testing process it done and we need to wait for the Schemathesis.io handler to finish
+    # It might still have some data to send
+    while queue.empty():
+        if time.monotonic() - start >= service.WORKER_FINISH_TIMEOUT:
+            return service.Timeout()
+        click.echo(f"{title}: {next(spinner)}\r", nl=False)
+        time.sleep(service.WORKER_CHECK_PERIOD)
+    return queue.get()
+
+
+def create_spinner(repetitions: int) -> Generator[str, None, None]:
+    """A simple spinner that yields its individual characters."""
+    assert repetitions > 0, "The number of repetitions should be greater than zero"
+    while True:
+        for ch in "⠋⠙⠹⠸⠼⠴⠦⠧⠇⠏":
+            # Skip branch coverage, as it is not possible because of the assertion above
+            for _ in range(repetitions):  # pragma: no branch
+                yield ch
+
 
 def display_checks_statistics(total: Dict[str, Dict[Union[str, Status], int]]) -> None:
     padding = 20
@@ -301,7 +408,7 @@ def display_internal_error(context: ExecutionContext, event: events.InternalErro
             f"Error: {message}\n"
             f"Add this option to your command line parameters to see full tracebacks: --show-errors-tracebacks"
         )
-        if event.exception_type == "jsonschema.exceptions.ValidationError":
+        if event.exception_type == "schemathesis.exceptions.SchemaLoadingError":
             message += "\n" + DISABLE_SCHEMA_VALIDATION_MESSAGE
         click.secho(message, fg="red")
 
@@ -320,9 +427,10 @@ def handle_initialized(context: ExecutionContext, event: events.Initialized) ->
     )
     click.echo(versions)
     click.echo(f"rootdir: {os.getcwd()}")
+    current_profile = settings._current_profile  # type: ignore[attr-defined]
     click.echo(
-        f"hypothesis profile '{settings._current_profile}' "  # type: ignore
-        f"-> {settings.get_profile(settings._current_profile).show_changed()}"
+        f"hypothesis profile '{current_profile}' "  # type: ignore
+        f"-> {settings.get_profile(current_profile).show_changed()}"
     )
     if event.location is not None:
         click.echo(f"Schema location: {event.location}")
@@ -330,6 +438,8 @@ def handle_initialized(context: ExecutionContext, event: events.Initialized) ->
     click.echo(f"Specification version: {event.specification_name}")
     click.echo(f"Workers: {context.workers_num}")
     click.secho(f"Collected API operations: {context.operations_count}", bold=True)
+    if context.service is not None:
+        click.secho("Schemathesis.io: ENABLED", bold=True)
     if context.operations_count >= 1:
         click.echo()
 
