@@ -1,4 +1,8 @@
 from enum import Enum
+from typing import List
+
+import pytest
+from packaging import version
 
 from ._compat import metadata
 
@@ -8,12 +12,14 @@ except metadata.PackageNotFoundError:
     # Local run without installation
     __version__ = "dev"
 
+IS_PYTEST_ABOVE_54 = version.parse(pytest.__version__) >= version.parse("5.4.0")
 
 USER_AGENT = f"schemathesis/{__version__}"
 # Maximum test running time
 DEFAULT_DEADLINE = 15000  # pragma: no mutate
 DEFAULT_RESPONSE_TIMEOUT = 10000  # pragma: no mutate
 DEFAULT_STATEFUL_RECURSION_LIMIT = 5  # pragma: no mutate
+HTTP_METHODS = frozenset({"get", "put", "post", "delete", "options", "head", "patch", "trace"})
 RECURSIVE_REFERENCE_ERROR_MESSAGE = (
     "Currently, Schemathesis can't generate data for this operation due to "
     "recursive references in the operation definition. See more information in "
@@ -38,6 +44,10 @@ class DataGenerationMethod(str, Enum):
     def default(cls) -> "DataGenerationMethod":
         return cls.positive
 
+    @classmethod
+    def all(cls) -> List["DataGenerationMethod"]:
+        return list(DataGenerationMethod)
+
     def as_short_name(self) -> str:
         return {
             DataGenerationMethod.positive: "P",
@@ -56,7 +66,7 @@ class CodeSampleStyle(str, Enum):
 
     @classmethod
     def default(cls) -> "CodeSampleStyle":
-        return cls.python
+        return cls.curl
 
     @classmethod
     def from_str(cls, value: str) -> "CodeSampleStyle":
