@@ -9,17 +9,17 @@ from _pytest.nodes import Node
 from _pytest.python import Class, Function, FunctionDefinition, Metafunc, Module, PyCollector
 from hypothesis.errors import InvalidArgument
 from hypothesis_jsonschema._canonicalise import HypothesisRefResolutionError
-from packaging import version
 
 from .. import DataGenerationMethod
 from .._hypothesis import create_test
-from ..constants import RECURSIVE_REFERENCE_ERROR_MESSAGE
+from ..constants import IS_PYTEST_ABOVE_54, RECURSIVE_REFERENCE_ERROR_MESSAGE
 from ..exceptions import InvalidSchema
 from ..models import APIOperation
 from ..utils import (
     PARAMETRIZE_MARKER,
     Ok,
     Result,
+    fail_on_no_matches,
     get_given_args,
     get_given_kwargs,
     is_given_applied,
@@ -28,13 +28,11 @@ from ..utils import (
     validate_given_args,
 )
 
-USE_FROM_PARENT = version.parse(pytest.__version__) >= version.parse("5.4.0")
-
 T = TypeVar("T", bound=Node)
 
 
 def create(cls: Type[T], *args: Any, **kwargs: Any) -> T:
-    if USE_FROM_PARENT:
+    if IS_PYTEST_ABOVE_54:
         return cls.from_parent(*args, **kwargs)  # type: ignore
     return cls(*args, **kwargs)
 
@@ -141,7 +139,7 @@ class SchemathesisCase(PyCollector):
                 data_generation_method=data_generation_method,
             )
         else:
-            fixtures.add_funcarg_pseudo_fixture_def(self.parent, metafunc, fixturemanager)
+            fixtures.add_funcarg_pseudo_fixture_def(self.parent, metafunc, fixturemanager)  # type: ignore[arg-type]
             fixtureinfo.prune_dependency_tree()
             for callspec in metafunc._calls:
                 subname = f"{name}[{callspec.id}]"
@@ -180,12 +178,15 @@ class SchemathesisCase(PyCollector):
     def collect(self) -> List[Function]:  # type: ignore
         """Generate different test items for all API operations available in the given schema."""
         try:
-            return [
+            items = [
                 item
                 for data_generation_method in self.schemathesis_case.data_generation_methods
                 for operation in self.schemathesis_case.get_all_operations()
                 for item in self._gen_items(operation, data_generation_method)
             ]
+            if not items:
+                fail_on_no_matches(self.nodeid)
+            return items
         except Exception:
             pytest.fail("Error during collection")
 
