@@ -34,6 +34,7 @@ import requests
 import werkzeug
 from hypothesis import event, note, reject
 from hypothesis import strategies as st
+from requests.structures import CaseInsensitiveDict
 from starlette.testclient import TestClient as ASGIClient
 
 from . import failures, serializers
@@ -49,7 +50,6 @@ from .exceptions import (
     FailureContext,
     InvalidSchema,
     SerializationNotPossible,
-    UsageError,
     get_grouped_exception,
     get_timeout_error,
 )
@@ -57,7 +57,14 @@ from .hooks import GLOBAL_HOOK_DISPATCHER, HookContext, HookDispatcher
 from .parameters import Parameter, ParameterSet, PayloadAlternatives
 from .serializers import Serializer, SerializerContext
 from .types import Body, Cookies, FormData, Headers, NotSet, PathParameters, Query
-from .utils import NOT_SET, GenericResponse, WSGIResponse, deprecated_property, get_response_payload
+from .utils import (
+    NOT_SET,
+    GenericResponse,
+    WSGIResponse,
+    deprecated_property,
+    get_response_payload,
+    maybe_set_assertion_message,
+)
 
 if TYPE_CHECKING:
     from .schemas import BaseSchema
@@ -86,7 +93,7 @@ class Case:  # pylint: disable=too-many-public-methods
 
     operation: "APIOperation" = attr.ib()  # pragma: no mutate
     path_parameters: Optional[PathParameters] = attr.ib(default=None)  # pragma: no mutate
-    headers: Optional[Headers] = attr.ib(default=None)  # pragma: no mutate
+    headers: Optional[CaseInsensitiveDict] = attr.ib(default=None)  # pragma: no mutate
     cookies: Optional[Cookies] = attr.ib(default=None)  # pragma: no mutate
     query: Optional[Query] = attr.ib(default=None)  # pragma: no mutate
     # By default, there is no body, but we can't use `None` as the default value because it clashes with `null`
@@ -214,7 +221,8 @@ class Case:  # pylint: disable=too-many-public-methods
         printed_kwargs = ", ".join(
             f"{key}={repr(value)}" for key, value in kwargs.items() if should_display(key, value)
         )
-        args_repr = f"'{kwargs['url']}'"
+        url = kwargs["url"].replace("'", "\\'")
+        args_repr = f"'{url}'"
         if printed_kwargs:
             args_repr += f", {printed_kwargs}"
         return f"requests.{method}({args_repr})"
@@ -245,12 +253,11 @@ class Case:  # pylint: disable=too-many-public-methods
                 )
         return base_url
 
-    def _get_headers(self, headers: Optional[Dict[str, str]] = None) -> Dict[str, Any]:
-        final_headers = self.headers.copy() if self.headers is not None else {}
+    def _get_headers(self, headers: Optional[Dict[str, str]] = None) -> CaseInsensitiveDict:
+        final_headers = self.headers.copy() if self.headers is not None else CaseInsensitiveDict()
         if headers:
             final_headers.update(headers)
-        if "user-agent" not in {header.lower() for header in final_headers}:
-            final_headers["User-Agent"] = USER_AGENT
+        final_headers.setdefault("User-Agent", USER_AGENT)
         return final_headers
 
     def _get_serializer(self) -> Optional[Serializer]:
@@ -288,8 +295,7 @@ class Case:  # pylint: disable=too-many-public-methods
         if additional_headers:
             # Additional headers, needed for the serializer
             for key, value in additional_headers.items():
-                if key.lower() not in {header.lower() for header in final_headers}:
-                    final_headers[key] = value
+                final_headers.setdefault(key, value)
         return {
             "method": self.method,
             "url": url,
@@ -344,7 +350,8 @@ class Case:  # pylint: disable=too-many-public-methods
         return {
             "method": self.method,
             "path": self.operation.schema.get_full_path(self.formatted_path),
-            "headers": final_headers,
+            # Convert to a regular dictionary, as we use `CaseInsensitiveDict` which is not supported by Werkzeug
+            "headers": dict(final_headers),
             "query_string": self.query,
             **extra,
         }
@@ -405,7 +412,8 @@ class Case:  # pylint: disable=too-many-public-methods
         for check in chain(checks, additional_checks):
             try:
                 check(response, self)
-            except CheckFailed as exc:
+            except AssertionError as exc:
+                maybe_set_assertion_message(exc, check.__name__)
                 errors.append(exc)
         if errors:
             exception_cls = get_grouped_exception(self.operation.verbose_name, *errors)
@@ -449,7 +457,7 @@ class Case:  # pylint: disable=too-many-public-methods
         kwargs = self.as_requests_kwargs(base_url)
         request = requests.Request(**kwargs)
         prepared = requests.Session().prepare_request(request)  # type: ignore
-        return prepared.url
+        return cast(str, prepared.url)
 
     def partial_deepcopy(self) -> "Case":
         return self.__class__(
@@ -526,7 +534,7 @@ class APIOperation(Generic[P, C]):
     cookies: ParameterSet[P] = attr.ib(factory=ParameterSet)  # pragma: no mutate
     query: ParameterSet[P] = attr.ib(factory=ParameterSet)  # pragma: no mutate
     body: PayloadAlternatives[P] = attr.ib(factory=PayloadAlternatives)  # pragma: no mutate
-    case_cls: Type[C] = attr.ib(default=Case)
+    case_cls: Type[C] = attr.ib(default=Case)  # type: ignore
 
     @verbose_name.default
     def _verbose_name_default(self) -> str:
@@ -647,20 +655,8 @@ class APIOperation(Generic[P, C]):
 
         The main use case is constructing Case instances completely manually, without data generation.
         """
-        if body is not NOT_SET and media_type is None:
-            # If the user wants to send payload, then there should be a media type, otherwise the payload is ignored
-            media_types = self.get_request_payload_content_types()
-            if len(media_types) == 1:
-                # The only available option
-                media_type = media_types[0]
-            else:
-                media_types_repr = ", ".join(media_types)
-                raise UsageError(
-                    "Can not detect appropriate media type. "
-                    "You can either specify one of the defined media types "
-                    f"or pass any other media type available for serialization. Defined media types: {media_types_repr}"
-                )
-        return self.case_cls(
+        return self.schema.make_case(
+            case_cls=self.case_cls,
             operation=self,
             path_parameters=path_parameters,
             headers=headers,
