@@ -16,7 +16,7 @@ from hypothesis.errors import HypothesisException, InvalidArgument
 from hypothesis_jsonschema._canonicalise import HypothesisRefResolutionError
 from requests.auth import HTTPDigestAuth, _basic_auth_str
 
-from ... import failures
+from ... import failures, hooks
 from ...constants import (
     DEFAULT_STATEFUL_RECURSION_LIMIT,
     RECURSIVE_REFERENCE_ERROR_MESSAGE,
@@ -37,8 +37,15 @@ from ...runner import events
 from ...schemas import BaseSchema
 from ...stateful import Feedback, Stateful
 from ...targets import Target, TargetContext
-from ...types import RawAuth
-from ...utils import GenericResponse, Ok, WSGIResponse, capture_hypothesis_output, format_exception
+from ...types import RawAuth, RequestCert
+from ...utils import (
+    GenericResponse,
+    Ok,
+    WSGIResponse,
+    capture_hypothesis_output,
+    format_exception,
+    maybe_set_assertion_message,
+)
 from ..serialization import SerializedTestResult
 
 
@@ -84,18 +91,24 @@ class BaseRunner:
             yield _finish()
             return
 
-        for event in self._execute(results):
-            yield event
-            if stop_event.is_set() or (
-                self.exit_first
-                and isinstance(event, events.AfterExecution)
-                and event.status in (Status.error, Status.failure)
-            ):
-                break
+        try:
+            for event in self._execute(results, stop_event):
+                yield event
+        except KeyboardInterrupt:
+            yield events.Interrupted()
 
         yield _finish()
 
-    def _execute(self, results: TestResultSet) -> Generator[events.ExecutionEvent, None, None]:
+    def _should_stop(self, event: events.ExecutionEvent) -> bool:
+        return (
+            self.exit_first
+            and isinstance(event, events.AfterExecution)
+            and event.status in (Status.error, Status.failure)
+        )
+
+    def _execute(
+        self, results: TestResultSet, stop_event: threading.Event
+    ) -> Generator[events.ExecutionEvent, None, None]:
         raise NotImplementedError
 
     def _run_tests(
@@ -193,14 +206,17 @@ def handle_schema_error(
             verbose_name=verbose_name,
             relative_path=error.path,
             recursion_level=recursion_level,
+            data_generation_method=data_generation_method,
             correlation_id=correlation_id,
         )
         yield events.AfterExecution(
             method=method,
             path=error.full_path,
             relative_path=error.path,
+            verbose_name=verbose_name,
             status=Status.error,
             result=SerializedTestResult.from_test_result(result),
+            data_generation_method=data_generation_method,
             elapsed_time=0.0,
             hypothesis_output=[],
             correlation_id=correlation_id,
@@ -234,7 +250,10 @@ def run_test(  # pylint: disable=too-many-locals
     # To simplify connecting `before` and `after` events in external systems
     correlation_id = uuid.uuid4().hex
     yield events.BeforeExecution.from_operation(
-        operation=operation, recursion_level=recursion_level, correlation_id=correlation_id
+        operation=operation,
+        recursion_level=recursion_level,
+        data_generation_method=data_generation_method,
+        correlation_id=correlation_id,
     )
     hypothesis_output: List[str] = []
     errors: List[Exception] = []
@@ -307,6 +326,7 @@ def run_test(  # pylint: disable=too-many-locals
         elapsed_time=test_elapsed_time,
         hypothesis_output=hypothesis_output,
         operation=operation,
+        data_generation_method=data_generation_method,
         correlation_id=correlation_id,
     )
 
@@ -374,10 +394,7 @@ def run_checks(
                 check_result = result.add_success(check_name, case, response, elapsed_time)
                 check_results.append(check_result)
         except AssertionError as exc:
-            message = str(exc)
-            if not message:
-                message = f"Check '{check_name}' failed"
-                exc.args = (message,)
+            message = maybe_set_assertion_message(exc, check_name)
             errors.append(exc)
             if isinstance(exc, CheckFailed):
                 context = exc.context
@@ -439,7 +456,7 @@ class ErrorCollector:
         return self
 
     # Typing: The return type suggested by mypy is `Literal[False]`, but I don't want to introduce dependency on the
-    # `typing_extensions` package for Python 3.6
+    # `typing_extensions` package for Python 3.7
     def __exit__(
         self, exc_type: Optional[Type[BaseException]], exc_val: Optional[BaseException], exc_tb: Optional[TracebackType]
     ) -> Any:
@@ -466,6 +483,7 @@ def network_test(
     session: requests.Session,
     request_timeout: Optional[int],
     request_tls_verify: bool,
+    request_cert: Optional[RequestCert],
     store_interactions: bool,
     headers: Optional[Dict[str, Any]],
     feedback: Feedback,
@@ -491,6 +509,7 @@ def network_test(
                 headers,
                 feedback,
                 request_tls_verify,
+                request_cert,
                 max_response_time,
             )
             add_cases(
@@ -506,6 +525,7 @@ def network_test(
                 headers,
                 feedback,
                 request_tls_verify,
+                request_cert,
                 max_response_time,
             )
 
@@ -521,11 +541,23 @@ def _network_test(
     headers: Optional[Dict[str, Any]],
     feedback: Feedback,
     request_tls_verify: bool,
+    request_cert: Optional[RequestCert],
     max_response_time: Optional[int],
 ) -> requests.Response:
     check_results: List[Check] = []
     try:
-        response = case.call(session=session, headers=headers, timeout=timeout, verify=request_tls_verify)
+        hook_context = HookContext(operation=case.operation)
+        hooks.dispatch("before_call", hook_context, case)
+        kwargs: Dict[str, Any] = {
+            "session": session,
+            "headers": headers,
+            "timeout": timeout,
+            "verify": request_tls_verify,
+            "cert": request_cert,
+        }
+        hooks.dispatch("process_call_kwargs", hook_context, case, kwargs)
+        response = case.call(**kwargs)
+        hooks.dispatch("after_call", hook_context, case, response)
     except CheckFailed as exc:
         check_name = "request_timeout"
         requests_kwargs = case.as_requests_kwargs(base_url=case.get_full_base_url(), headers=headers)
@@ -613,7 +645,12 @@ def _wsgi_test(
 ) -> WSGIResponse:
     with catching_logs(LogCaptureHandler(), level=logging.DEBUG) as recorded:
         start = time.monotonic()
-        response = case.call_wsgi(headers=headers)
+        hook_context = HookContext(operation=case.operation)
+        hooks.dispatch("before_call", hook_context, case)
+        kwargs = {"headers": headers}
+        hooks.dispatch("process_call_kwargs", hook_context, case, kwargs)
+        response = case.call_wsgi(**kwargs)
+        hooks.dispatch("after_call", hook_context, case, response)
         elapsed = time.monotonic() - start
     context = TargetContext(case=case, response=response, response_time=elapsed)
     run_targets(targets, context)
@@ -696,7 +733,12 @@ def _asgi_test(
     feedback: Feedback,
     max_response_time: Optional[int],
 ) -> requests.Response:
-    response = case.call_asgi(headers=headers)
+    hook_context = HookContext(operation=case.operation)
+    hooks.dispatch("before_call", hook_context, case)
+    kwargs: Dict[str, Any] = {"headers": headers}
+    hooks.dispatch("process_call_kwargs", hook_context, case, kwargs)
+    response = case.call_asgi(**kwargs)
+    hooks.dispatch("after_call", hook_context, case, response)
     context = TargetContext(case=case, response=response, response_time=response.elapsed.total_seconds())
     run_targets(targets, context)
     status = Status.success
