@@ -11,6 +11,7 @@ from weakref import WeakKeyDictionary
 from hypothesis import strategies as st
 from hypothesis_jsonschema import from_schema
 from requests.auth import _basic_auth_str
+from requests.structures import CaseInsensitiveDict
 
 from ... import utils
 from ...constants import DataGenerationMethod
@@ -24,9 +25,10 @@ from .negative import negative_schema
 from .parameters import OpenAPIBody, parameters_to_json_schema
 from .utils import is_header_location
 
+HEADER_FORMAT = "_header_value"
 PARAMETERS = frozenset(("path_parameters", "headers", "cookies", "query", "body"))
 SLASH = "/"
-STRING_FORMATS = {}
+STRING_FORMATS: Dict[str, st.SearchStrategy] = {}
 StrategyFactory = Callable[[Dict[str, Any], str, str, Optional[str]], st.SearchStrategy]
 
 
@@ -62,7 +64,7 @@ def init_default_strategies() -> None:
     # Define valid characters here to avoid filtering them out in `is_valid_header` later
     header_value = st.text(alphabet=st.characters(min_codepoint=0, max_codepoint=255, blacklist_characters="\n\r"))
     # Header values with leading non-visible chars can't be sent with `requests`
-    register_string_format("_header_value", header_value.map(str.lstrip))
+    register_string_format(HEADER_FORMAT, header_value.map(str.lstrip))
     register_string_format("_basic_auth", st.tuples(latin1_text, latin1_text).map(make_basic_auth_str))  # type: ignore
     register_string_format(
         "_bearer_auth",
@@ -119,18 +121,17 @@ def get_case_strategy(  # pylint: disable=too-many-locals
     The primary purpose of this behavior is to prevent sending incomplete explicit examples by generating missing parts
     as it works with `body`.
     """
-    to_strategy = {
-        DataGenerationMethod.positive: make_positive_strategy,
-        DataGenerationMethod.negative: make_negative_strategy,
-    }[data_generation_method]
+    to_strategy = DATA_GENERATION_METHOD_TO_STRATEGY_FACTORY[data_generation_method]
 
     context = HookContext(operation)
 
     with detect_invalid_schema(operation):
-        path_parameters = get_parameters_value(path_parameters, "path", draw, operation, context, hooks, to_strategy)
-        headers = get_parameters_value(headers, "header", draw, operation, context, hooks, to_strategy)
-        cookies = get_parameters_value(cookies, "cookie", draw, operation, context, hooks, to_strategy)
-        query = get_parameters_value(query, "query", draw, operation, context, hooks, to_strategy)
+        path_parameters_value = get_parameters_value(
+            path_parameters, "path", draw, operation, context, hooks, to_strategy
+        )
+        headers_value = get_parameters_value(headers, "header", draw, operation, context, hooks, to_strategy)
+        cookies_value = get_parameters_value(cookies, "cookie", draw, operation, context, hooks, to_strategy)
+        query_value = get_parameters_value(query, "query", draw, operation, context, hooks, to_strategy)
 
         media_type = None
         if body is NOT_SET:
@@ -154,10 +155,10 @@ def get_case_strategy(  # pylint: disable=too-many-locals
     return Case(
         operation=operation,
         media_type=media_type,
-        path_parameters=path_parameters,
-        headers=headers,
-        cookies=cookies,
-        query=query,
+        path_parameters=path_parameters_value,
+        headers=CaseInsensitiveDict(headers_value) if headers_value is not None else headers_value,
+        cookies=cookies_value,
+        query=query_value,
         body=body,
         data_generation_method=data_generation_method,
     )
@@ -224,7 +225,7 @@ def get_parameters_value(
     context: HookContext,
     hooks: Optional[HookDispatcher],
     to_strategy: StrategyFactory,
-) -> Dict[str, Any]:
+) -> Optional[Dict[str, Any]]:
     """Get the final value for the specified location.
 
     If the value is not set, then generate it from the relevant strategy. Otherwise, check what is missing in it and
@@ -280,7 +281,9 @@ def get_parameters_strategy(
             "cookie": is_valid_header,
             "query": is_valid_query,
         }[location]
-        strategy = strategy.filter(filter_func)
+        # Headers with special format do not need filtration
+        if not (is_header_location(location) and _can_skip_header_filter(schema)):
+            strategy = strategy.filter(filter_func)
         # Path & query parameters will be cast to string anyway, but having their JSON equivalents for
         # `True` / `False` / `None` improves chances of them passing validation in apps that expect boolean / null types
         # and not aware of Python-specific representation of those types
@@ -323,11 +326,16 @@ def make_positive_strategy(
         # This way, only allowed values will be used during data generation, which reduces the amount of filtering later
         # If a property schema contains `pattern` it leads to heavy filtering and worse performance - therefore, skip it
         for sub_schema in schema.get("properties", {}).values():
-            if len(sub_schema) == 1 and "type" in sub_schema:
-                sub_schema.setdefault("format", "_header_value")
+            if list(sub_schema) == ["type"]:
+                sub_schema.setdefault("format", HEADER_FORMAT)
     return from_schema(schema, custom_formats=STRING_FORMATS)
 
 
+def _can_skip_header_filter(schema: Dict[str, Any]) -> bool:
+    # All headers should contain HEADER_FORMAT in order to avoid header filter
+    return all(sub_schema.get("format") == HEADER_FORMAT for sub_schema in schema.get("properties", {}).values())
+
+
 def make_negative_strategy(
     schema: Dict[str, Any], operation_name: str, location: str, media_type: Optional[str]
 ) -> st.SearchStrategy:
@@ -336,6 +344,12 @@ def make_negative_strategy(
     )
 
 
+DATA_GENERATION_METHOD_TO_STRATEGY_FACTORY = {
+    DataGenerationMethod.positive: make_positive_strategy,
+    DataGenerationMethod.negative: make_negative_strategy,
+}
+
+
 def is_valid_path(parameters: Dict[str, Any]) -> bool:
     """Empty strings ("") are excluded from path by urllib3.
 
