@@ -1,5 +1,5 @@
 import json
-from typing import Any, ClassVar, Dict, Iterable, List, Tuple
+from typing import Any, ClassVar, Dict, Iterable, List, Optional, Tuple
 
 import attr
 
@@ -16,6 +16,11 @@ class OpenAPIParameter(Parameter):
     nullable_field: ClassVar[str]
     supported_jsonschema_keywords: ClassVar[Tuple[str, ...]]
 
+    @property
+    def description(self) -> Optional[str]:
+        """A brief parameter description."""
+        return self.definition.get("description")
+
     @property
     def example(self) -> Any:
         """The primary example defined for this parameter."""
@@ -235,7 +240,6 @@ class OpenAPI20Body(OpenAPIBody, OpenAPI20Parameter):
         "$ref",
         "format",
         "multipleOf",
-        "multipleOf",
         "maximum",
         "exclusiveMaximum",
         "minimum",
@@ -285,6 +289,7 @@ class OpenAPI30Body(OpenAPIBody, OpenAPI30Parameter):
     # The `required` keyword is located above the schema for concrete media-type;
     # Therefore, it is passed here explicitly
     required: bool = attr.ib(default=False)
+    description: Optional[str] = attr.ib(default=None)
 
     def as_json_schema(self) -> Dict[str, Any]:
         """Convert body definition to JSON Schema."""
@@ -312,7 +317,7 @@ class OpenAPI30Body(OpenAPIBody, OpenAPI30Parameter):
 class OpenAPI20CompositeBody(OpenAPIBody, OpenAPI20Parameter):
     """A special container to abstract over multiple `formData` parameters."""
 
-    definition: List[OpenAPIParameter] = attr.ib()
+    definition: List[OpenAPI20Parameter] = attr.ib()
 
     @classmethod
     def from_parameters(cls, *parameters: Dict[str, Any], media_type: str) -> "OpenAPI20CompositeBody":
@@ -321,6 +326,10 @@ class OpenAPI20CompositeBody(OpenAPIBody, OpenAPI20Parameter):
             media_type=media_type,
         )
 
+    @property
+    def description(self) -> Optional[str]:
+        return None
+
     @property
     def is_required(self) -> bool:
         # We generate an object for formData - it is always required.
