@@ -25,6 +25,7 @@ from typing import (
     overload,
 )
 
+import pytest
 import requests
 import yaml
 import yarl
@@ -38,9 +39,9 @@ from requests.utils import check_header_validity  # type: ignore
 from werkzeug.wrappers import Response as BaseResponse
 from werkzeug.wrappers.json import JSONMixin
 
-from .constants import USER_AGENT
+from .constants import USER_AGENT, DataGenerationMethod
 from .exceptions import UsageError
-from .types import Filter, GenericTest, NotSet, RawAuth
+from .types import DataGenerationMethodInput, Filter, GenericTest, NotSet, RawAuth
 
 try:
     from yaml import CSafeLoader as SafeLoader
@@ -93,6 +94,10 @@ def is_schemathesis_test(func: Callable) -> bool:
         return False
 
 
+def fail_on_no_matches(node_id: str) -> NoReturn:  # type: ignore
+    pytest.fail(f"Test function {node_id} does not match any API operations and therefore has no effect")
+
+
 def force_tuple(item: Filter) -> Union[List, Set, Tuple]:
     if not isinstance(item, (list, set, tuple)):
         return (item,)
@@ -415,3 +420,17 @@ def compose(*functions: Callable) -> Callable:
         return x
 
     return functools.reduce(lambda f, g: lambda x: f(g(x)), functions, noop)
+
+
+def maybe_set_assertion_message(exc: AssertionError, check_name: str) -> str:
+    message = str(exc)
+    if not message:
+        message = f"Check '{check_name}' failed"
+        exc.args = (message,)
+    return message
+
+
+def prepare_data_generation_methods(data_generation_methods: DataGenerationMethodInput) -> List[DataGenerationMethod]:
+    if isinstance(data_generation_methods, DataGenerationMethod):
+        return [data_generation_methods]
+    return list(data_generation_methods)
